@@ -1,25 +1,25 @@
 /*
- PrefixSpan: An efficient algorithm for sequential pattern mining
+  PrefixSpan: An efficient algorithm for sequential pattern mining
 
- $Id: prefixspan.cpp,v 1.8 2002/04/03 13:35:23 taku-ku Exp $;
+  $Id: prefixspan.cpp,v 1.8 2002/04/03 13:35:23 taku-ku Exp $;
 
- Copyright (C) 2002 Taku Kudo  All rights reserved.
- This is free software with ABSOLUTELY NO WARRANTY.
+  Copyright (C) 2002 Taku Kudo  All rights reserved.
+  This is free software with ABSOLUTELY NO WARRANTY.
 
- This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
- (at your option) any later version.
+  This program is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; either version 2 of the License, or
+  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- 02111-1307, USA
+  You should have received a copy of the GNU General Public License
+  along with this program; if not, write to the Free Software
+  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+  02111-1307, USA
 */
 
 #include <iostream>
@@ -71,9 +71,9 @@ private:
 	  *os << (i ? " " : "") << pattern[i].first;
       } else {
 	*os << "<what>";
-	 for (unsigned int i = 0; i < pattern.size(); i++)
-	   *os << (i ? " " : "") << pattern[i].first 
-	       << delimiter << pattern[i].second;
+	for (unsigned int i = 0; i < pattern.size(); i++)
+	  *os << (i ? " " : "") << pattern[i].first 
+	      << delimiter << pattern[i].second;
       }
 
       *os << "</what>" << endl;
@@ -91,8 +91,13 @@ private:
 	// print found pattern only
 	if (all) {
 	  *os << pattern[pattern.size()-1].second;
-	  for (unsigned int i = 0; i < pattern.size(); i++)
-	    *os << " " << pattern[i].first;
+	  for (unsigned int i = 0; i < pattern.size(); i++) {
+	    if (pattern[i].second == 0) {
+	      *os << " " << delimiter << delimiter;
+	    } else {
+	      *os << " " << pattern[i].first;
+	    }
+	  }
 	} else {
 	  
 	  skipped = false;
@@ -110,8 +115,7 @@ private:
 	  
 	}
 	*os << endl;
-	//	*os << skipcount;
-	//	*os << endl;
+
       }
     }
   }
@@ -136,14 +140,14 @@ private:
 	      &&(ntmp.find (item) == ntmp.end())) {
 	    ntmp[item] = j ;
 	  }
-	 }
+	}
       } else {
 	// neighbor
 	unsigned int j = pos + 1;
 	if (j < size) {
 	  T item = transaction[id][j];	
 	  if ((stopwordset.find(item) == stopwordset.end()) &&
-	    (ntmp.find (item) == ntmp.end())) {
+	      (ntmp.find (item) == ntmp.end())) {
 	    ntmp[item] = j ;
 	  }
 	}
@@ -164,9 +168,9 @@ private:
       for (typename map <T, int>::iterator k = stmp.begin(); k != stmp.end(); ++k) 
 	scounter[k->first].push_back (make_pair <unsigned int, int> (id, k->second));
 
-  }
+    }
     // for root or neighbor
-  for (typename map <T, vector <pair <unsigned int, int> > >::iterator l = ncounter.begin (); 
+    for (typename map <T, vector <pair <unsigned int, int> > >::iterator l = ncounter.begin (); 
 	 l != ncounter.end (); ) {
       if (l->second.size() < minsup) {
 	typename map <T, vector <pair <unsigned int, int> > >::iterator tmp = l;
@@ -260,10 +264,10 @@ public:
     }
 
     while (getline (is, line)) {
-       tmp.clear ();
-       istrstream istrs ((char *)line.c_str());
-       while (istrs >> item) tmp.push_back (item);
-       transaction.push_back (tmp);
+      tmp.clear ();
+      istrstream istrs ((char *)line.c_str());
+      while (istrs >> item) tmp.push_back (item);
+      transaction.push_back (tmp);
     }
     
     return is;
@@ -358,24 +362,24 @@ int main (int argc, char **argv)
   }
    
   if (type == "int") { 
-     PrefixSpan<unsigned int> prefixspan (minsup, minpat, maxpat, mingap, maxgap, minskip, maxskip, all, where, delimiter, stopwordfile, stopword, verbose);
-     prefixspan.read (cin);
-     prefixspan.run  (cout);
+    PrefixSpan<unsigned int> prefixspan (minsup, minpat, maxpat, mingap, maxgap, minskip, maxskip, all, where, delimiter, stopwordfile, stopword, verbose);
+    prefixspan.read (cin);
+    prefixspan.run  (cout);
   }else if (type == "short") {
-     PrefixSpan<unsigned short> prefixspan (minsup, minpat, maxpat, mingap, maxgap, minskip, maxskip, all, where, delimiter, stopwordfile, stopword, verbose);
-     prefixspan.read (cin);
-     prefixspan.run  (cout);
+    PrefixSpan<unsigned short> prefixspan (minsup, minpat, maxpat, mingap, maxgap, minskip, maxskip, all, where, delimiter, stopwordfile, stopword, verbose);
+    prefixspan.read (cin);
+    prefixspan.run  (cout);
   } else if (type == "char") {
-     PrefixSpan<unsigned char> prefixspan (minsup, minpat, maxpat, mingap, maxgap, minskip, maxskip, all, where, delimiter, stopwordfile, stopword, verbose);
-     prefixspan.read (cin);
-     prefixspan.run  (cout);
+    PrefixSpan<unsigned char> prefixspan (minsup, minpat, maxpat, mingap, maxgap, minskip, maxskip, all, where, delimiter, stopwordfile, stopword, verbose);
+    prefixspan.read (cin);
+    prefixspan.run  (cout);
   } else if (type == "string") {
-     PrefixSpan<string> prefixspan (minsup, minpat, maxpat, mingap, maxgap, minskip, maxskip, all, where, delimiter, stopwordfile, stopword, verbose);
-     prefixspan.read (cin);
-     prefixspan.run  (cout);
+    PrefixSpan<string> prefixspan (minsup, minpat, maxpat, mingap, maxgap, minskip, maxskip, all, where, delimiter, stopwordfile, stopword, verbose);
+    prefixspan.read (cin);
+    prefixspan.run  (cout);
   } else { 
-     cerr << "Unknown Item Type: " << type << " : choose from [string|int|short|char]" << endl;
-     return -1;
+    cerr << "Unknown Item Type: " << type << " : choose from [string|int|short|char]" << endl;
+    return -1;
   }
 
   return 0;
